import re

filename_pattern = re.compile(r'(\/.*?\.[\w:]+)')
url_pattern = re.compile(r'https?://(?:[-\w.]|(?:%[\da-fA-F]{2}))+')
email_pattern = re.compile(r'^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$')
file_extension_pattern = re.compile(r'\.(?P<ext>[a-zA-Z0-9]+$)')


def is_url(string):
    return url_pattern.match(string) is not None


def is_email(string):
    return email_pattern.match(string) is not None


def is_filename(string):
    return filename_pattern.match(string) is not None


def find_all_filename(string):
    return re.findall(filename_pattern, string)


def get_file_extension(string):
    return re.search(file_extension_pattern, string).group('ext')



import collections
from collections.abc import Iterable


def wrap_list(some):
    if isiter(some, allow_dict=False):
        return some
    return [some]


def isiter(arg, allow_dict=True, allow_str=False):
    if not isinstance(arg, Iterable):
        return False
    elif type(arg) is str and not allow_str:
        return False
    elif type(arg) is dict and not allow_dict:
        return False
    else:
        return True


def chunk_iter(iterable, size):
    arr = []
    for some in iterable:
        arr.append(some)
        if len(arr) >= size:
            yield arr
            arr = []
    if len(arr) > 0:
        yield arr


class MutableMapping(collections.MutableMapping):
    """A dictionary that applies an arbitrary key-altering
       function before accessing the keys"""

    def __init__(self, *args, **kwargs):
        self.store = dict()
        self.update(dict(*args, **kwargs))  # use the free update to set keys

    def __getitem__(self, key):
        return self.store[self.__keytransform__(key)]

    def __setitem__(self, key, value):
        self.store[self.__keytransform__(key)] = value

    def __delitem__(self, key):
        del self.store[self.__keytransform__(key)]

    def __iter__(self):
        return iter(self.store)

    def __len__(self):
        return len(self.store)

    def __keytransform__(self, key):
        return key



def raise_or(e, value, throw=False):
    if throw:
        raise e
    return value


def safeaccess(obj, *args, context=None, default=None):
    if obj is None:
        return default

    context = context or dict()
    value = obj

    for arg in args:
        value = getattr(value, arg)
        if value is None:
            return default
        if callable(value):
            value = value(**context)

    return value



import re

snake_first = re.compile('(.)([A-Z][a-z]+)')
snake_second = re.compile('([a-z0-9])([A-Z])')


def snake_to_camel(s):
    components = s.split('_')
    return ''.join(x.title() for x in components)


def camel_to_snake(s):
    s1 = re.sub(snake_first, r'\1_\2', s)
    return re.sub(snake_second, r'\1_\2', s1).lower()



import decimal


def half_round(num, fmt):
    """
    decimal_round(0.1235, '1.0') == 0.0
    decimal_round(0.1235, '0.1') == 0.1
    decimal_round(0.1235, '0.01') == 0.12
    decimal_round(0.1235, '0.001') == 0.124
    decimal_round(0.1235, '0.0001') == 0.1235
    """
    if fmt.find('.') >= 0:
        # Float
        n = decimal.Decimal(str(num)).quantize(decimal.Decimal(fmt), rounding=decimal.ROUND_HALF_UP)
        return float(n)
    else:
        # Integer
        raise ValueError('Format not allowed')


def clear_int(num, clear_with):
    clear_with = int('1'.zfill(intlen(clear_with)))

    if intlen(num) == intlen(clear_with):
        raise ValueError('You can not clear same length integer')
    return num // clear_with * clear_with


def intlen(num):
    return len(str(num))


def num_to_percent(part, whole):
    if whole == 0:
        return 0
    elif part > whole:
        raise ValueError('part must be less than whole.')
    return 100 * float(part) / float(whole)


def percent_to_num(whole, percent):
    if whole == 0:
        return 0
    return float(percent * whole) / 100


def isint(some):
    try:
        int(some)
        return True
    except ValueError:
        return False
